<?php

declare(strict_types=1);

namespace MNS\Jaayegah;

class RestApi
{

	public function __construct()
	{

		$this->init_hooks();
	}

	protected function init_hooks()
	{

		add_action('rest_api_init', [$this, 'register_routes']);
	}

	public function register_routes()
	{

		$this->register_admin_endpoints();
		$this->register_area_endpoints();
		$this->register_checkout_endpoints();
		$this->register_heatmap_endpoints();
		$this->register_location_endpoints();
		$this->register_order_endpoints();
		$this->register_postman_endpoints();
		$this->register_tapin_endpoints();
	}

	/**
	 * Admin API
	 *
	 * @return void
	 */
	public function register_admin_endpoints()
	{

		register_rest_route('mnsjay/admin/v1', '/rate', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'admin_rate'],
			'args' => [
				'action' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
				]
			],
			'permission_callback' => function () {
				return current_user_can('manage_options');
			}
		]);
	}

	/**
	 * Location API
	 *
	 * @return void
	 */
	public function register_location_endpoints()
	{

		register_rest_route('mnsjay/location/v1', '/submit/coords/(?P<coords>[\S]+)', array(
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'location_submit_coords'],
			'args' => array(
				'coords' => array(
					'default' => mnsjay_get_default_location(),
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => 'mnsjay_coords_validate'
				),
			),
			'permission_callback' => function () {
				return true;
			}
		));

		register_rest_route('mnsjay/location/v1', '/search', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'location_search'],
			'args' => [
				'coords' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return mnsjay_coords_validate($value);
					}
				],
				'query' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field'
				]
			],
			'permission_callback' => function () {
				return true;
			}
		]);
	}

	/**
	 * Area API
	 *
	 * @return void
	 */
	public function register_area_endpoints()
	{

		register_rest_route('mnsjay/area/v1', '/search', array(
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'area_search'],
			'args' => array(
				'country' => array(
					'default' => '',
					'required' => false,
					'sanitize_callback' => 'sanitize_text_field'
				),
				'province' => array(
					'default' => '',
					'required' => false,
					'sanitize_callback' => 'sanitize_text_field'
				),
				'county' => array(
					'default' => '',
					'required' => false,
					'sanitize_callback' => 'sanitize_text_field'
				),
				'city' => array(
					'default' => '',
					'required' => false,
					'sanitize_callback' => 'sanitize_text_field'
				),
			),
			'permission_callback' => function () {
				return true;
			}
		));
	}

	/**
	 * Order API
	 *
	 * @return void
	 */
	public function register_order_endpoints()
	{

		register_rest_route('mnsjay/order/v1', '/submit/coords', array(
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'order_submit_coords'],
			'args' => array(
				'coords' => array(
					'default' => mnsjay_get_default_location(),
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => 'mnsjay_coords_validate'
				),
				'address_type' => array(
					'default' => 'billing',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return in_array($value, ['billing', 'shipping']);
					}
				),
				'order_id' => array(
					'default' => 0,
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return get_post_type($value) === 'shop_order';
					}
				)
			),
			'permission_callback' => function () {
				return true;
			}
		));
	}

	/**
	 * Tapin API
	 *
	 * @return void
	 */
	public function register_tapin_endpoints()
	{

		register_rest_route('mnsjay/tapin/v1', '/register', array(
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'tapin_register'],
			'args' => array(
				'order_id' => array(
					'default' => 0,
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return get_post_type($value) === 'shop_order';
					}
				),
				'weight' => array(
					'default' => '',
					'required' => false,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return (is_numeric($value) or empty($value));
					}
				),
				'content_type' => array(
					'default' => 1,
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value);
					}
				)
			),
			'permission_callback' => function () {
				return true;
			}
		));

		register_rest_route('mnsjay/tapin/v1', '/change-status', array(
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'tapin_change_status'],
			'args' => array(
				'order_id' => array(
					'default' => 0,
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return get_post_type($value) === 'shop_order';
					}
				),
				'status' => array(
					'default' => 0,
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_int($value);
					}
				)
			),
			'permission_callback' => function () {
				return true;
			}
		));
	}

	/**
	 * Checkout API
	 *
	 * @return void
	 */
	public function register_checkout_endpoints()
	{

		register_rest_route('mnsjay/checkout/v1', '/geocode', array(
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'checkout_geocode'],
			'args' => array(
				'province' => array(
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field'
				),
				'city' => array(
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field'
				)
			),
			'permission_callback' => function () {
				return true;
			}
		));

		register_rest_route('mnsjay/checkout/v1', '/set-coords', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'checkout_set_coords'],
			'args' => [
				'coords' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return mnsjay_coords_validate($value);
					}
				],
				'checkout' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return in_array($value, ['shipping', 'billing']);
					}
				],
				'set' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => function ($value) {
						return boolval($value);
					},
					'validate_callback' => function ($value) {
						return is_bool($value);
					}
				]
			],
			'permission_callback' => function () {
				return true;
			}
		]);
	}

	/**
	 * Heatmap API
	 *
	 * @return void
	 */
	public function register_heatmap_endpoints()
	{

		register_rest_route('mnsjay/heatmap/v1', '/orders', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'heatmap_orders_get'],
			'args' => [
				'interval' => [
					'default' => 7,
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value);
					}
				],
			],
			'permission_callback' => function () {
				return mnsjay_is_current_user_in_roles(mnsjay_get_option_heatmap_roles());
			}
		]);
	}

	/**
	 * Postman API
	 *
	 * @return void
	 */
	public function register_postman_endpoints()
	{

		register_rest_route('mnsjay/postman/v1', '/delivery', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'postman_delivery'],
			'args' => [
				'delivery_code' => [
					'default' => '',
					'required' => false,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value) or empty($value);
					}
				],
				'order_id' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value);
					}
				]
			],
			'permission_callback' => function () {
				return mnsjay_is_user_postman(get_current_user_id());
			}
		]);

		register_rest_route('mnsjay/postman/v1', '/return', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'postman_return'],
			'args' => [
				'note' => [
					'default' => '',
					'required' => false,
					'sanitize_callback' => 'sanitize_textarea_field',
					'validate_callback' => function ($value) {
						return true;
					}
				],
				'order_id' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value);
					}
				]
			],
			'permission_callback' => function () {
				return mnsjay_is_user_postman(get_current_user_id());
			}
		]);

		register_rest_route('mnsjay/postman/v1', '/navigation', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'postman_navigation'],
			'args' => [
				'order_id' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value);
					}
				],
				'type' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return in_array($value, ['live-to-origin', 'live-to-destination', 'origin-to-destination']);
					}
				]
			],
			'permission_callback' => function () {
				return mnsjay_is_user_postman(get_current_user_id());
			}
		]);

		register_rest_route('mnsjay/postman/v1', '/tsp', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'postman_tsp'],
			'args' => [
				'origin' => [
					'default' => 'live',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return $value === 'live' or mnsjay_shipping_is_available($value);
					}
				],
				'destination' => [
					'default' => 'last',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return in_array($value, ['live', 'last']) or mnsjay_shipping_is_available($value);
					}
				],
				'pick_ups' => [
					'default' => [],
					'required' => true,
					'sanitize_callback' => function ($value) {
						return array_map('sanitize_text_field', $value);
					},
					'validate_callback' => function ($value) {
						return !empty($value);
					}
				],
				'drop_offs' => [
					'default' => [],
					'required' => true,
					'sanitize_callback' => function ($value) {
						return array_map('intval', $value);
					},
					'validate_callback' => function ($value) {
						return !empty($value);
					}
				],
				'postman_id' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value);
					}
				]
			],
			'permission_callback' => function () {
				return mnsjay_is_user_postman(get_current_user_id());
			}
		]);

		register_rest_route('mnsjay/postman/v1', '/live', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'postman_live'],
			'args' => [
				'coords' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return true;
					}
				],
				'postman_id' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value);
					}
				]
			],
			'permission_callback' => function () {
				return mnsjay_is_user_postman(get_current_user_id());
			}
		]);

		register_rest_route('mnsjay/postman/v1', '/settings', [
			'methods' => \WP_REST_Server::CREATABLE,
			'callback' => [$this, 'postman_settings'],
			'args' => [
				'inputs' => [
					'default' => [],
					'required' => true,
					'sanitize_callback' => function ($value) {
						return array_map('sanitize_text_field', $value);
					},
					'validate_callback' => function ($value) {
						return !empty($value);
					}
				],
				'postman_id' => [
					'default' => '',
					'required' => true,
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => function ($value) {
						return is_numeric($value);
					}
				]
			],
			'permission_callback' => function () {
				return mnsjay_is_user_postman(get_current_user_id());
			}
		]);
	}

	public function admin_rate($request)
	{

		$parameters = $request->get_params();

		switch ($parameters['action']) {
			case 'reminder':
				delete_user_meta(get_current_user_id(), '_mnsjay_user_rating_notice_info_demiss');
				update_user_meta(get_current_user_id(), '_mnsjay_user_rating_notice_info_reminder_time', time());
				break;
			case 'demiss':
				delete_user_meta(get_current_user_id(), '_mnsjay_user_rating_notice_info_reminder_time');
				update_user_meta(get_current_user_id(), '_mnsjay_user_rating_notice_info_demiss', true);
				break;
		}

		return new \WP_REST_Response(['status' => true], 200);
	}

	public function location_submit_coords($request)
	{

		$args = $request->get_params();

		$coords = mnsjay_coords_parse($args['coords']);

		$place = mnsjay_get_place($coords['lat'], $coords['lng']);

		if ($place['status']) {
			return new \WP_REST_Response($place, 200);
		} else {
			return new \WP_Error('webservice-error', $place['message']);
		}
	}

	public function location_search($request)
	{

		$parameters = $request->get_params();

		$coords = mnsjay_coords_parse($parameters['coords']);

		$results = mnsjay_search_places($parameters['query'], $coords['lat'], $coords['lng']);

		if ($results['status']) {
			return new \WP_REST_Response($results, 200);
		} else {
			return new \WP_Error('webservice-error', $results['message'], $results);
		}
	}

	public function area_search($request)
	{

		$args = $request->get_params();

		$search_queries = [
			'country' => isset($args['country']) ? $args['country'] : '',
			'province' => isset($args['province']) ? $args['province'] : '',
			'county' => isset($args['county']) ? $args['county'] : '',
			'city' => isset($args['city']) ? $args['city'] : ''
		];

		$results = mnsjay_area_search($search_queries);

		if ($results['status']) {
			return new \WP_REST_Response($results, 200);
		} else {
			return new \WP_Error('webservice-error', $results['message']);
		}
	}

	public function order_submit_coords($request)
	{

		$parameters = $request->get_params();

		$coords = mnsjay_coords_parse($parameters['coords']);
		$address_type = $parameters['address_type'];
		$order_id = $parameters['order_id'];

		$place = mnsjay_get_place($coords['lat'], $coords['lng']);

		if (!$place['status'])
			return new \WP_Error('webservice-error', $place['message']);

		mnsjay_update_order_location($order_id, $address_type, $parameters['coords']);

		return new \WP_REST_Response($place, 200);
	}

	public function tapin_register($request)
	{

		$parameters = $request->get_params();

		$order = wc_get_order($parameters['order_id']);
		$packaged = 'wc-mnsjay-packaged';
		$tapin = new \MNSJAY_Tapin($order);

		$tapin->order_set_meta([
			'custom_content_type' => $parameters['content_type'],
			'custom_package_weight' => $parameters['weight'],
		]);

		$response = $tapin->register_order();

		if (is_wp_error($response))
			return new \WP_Error('webservice-error', $response->get_error_message());

		$body = json_decode(wp_remote_retrieve_body($response), true);

		if (!isset($body['returns']['status']) or $body['returns']['status'] != 200)
			return new \WP_Error('webservice-error', $body['returns']['message']);

		$tapin->order_set_meta($body['entries']);

		$note = sprintf(
			__("Postal barcode of your shipment: %s\nYou can track your shipment by the link %s.", 'mnsjay'),
			$body['entries']['barcode'],
			'https://tracking.post.ir/'
		);

		$order->set_status($packaged, sprintf(__('Postal barcode: %s', 'mnsjay'), $body['entries']['barcode']));
		$order->add_order_note($note, true);
		$order->save();

		do_action('mnsjay_tapin_after_register', $tapin, $body['entries']);

		return new \WP_REST_Response([
			'message' => sanitize_text_field($body['returns']['message']),
		], 200);
	}

	public function tapin_change_status($request)
	{

		$parameters = $request->get_params();

		$order = wc_get_order($parameters['order_id']);
		$packaged = 'wc-mnsjay-packaged';

		if (!$order->has_status($packaged))
			return new \WP_Error('webservice-error', sprintf(__('This order is not in %s status', 'mnsjay'), wc_get_order_status_name($packaged)));

		$tapin = new \MNSJAY_Tapin($order);

		$response = $tapin->change_status($parameters['status']);

		if (is_wp_error($response))
			return new \WP_Error('webservice-error', $response->get_error_message());

		$body = json_decode(wp_remote_retrieve_body($response), true);

		if (!isset($body['returns']['status']) or $body['returns']['status'] != 200)
			return new \WP_Error('webservice-error', $body['returns']['message']);

		$order->set_status($tapin->woo_equal('status'));
		$order->save();

		do_action('mnsjay_tapin_after_change_status', $tapin, $parameters['status'], $body['entries']);

		return new \WP_REST_Response([
			'message' => sanitize_text_field($body['returns']['message']),
		], 200);
	}

	public function checkout_geocode($request)
	{

		$parameters = $request->get_params();

		$args = [
			'province' => $parameters['province'],
			'city' => $parameters['city']
		];

		$coords = mnsjay_geocode($args);

		if (!$coords['status'])
			return new \WP_Error('webservice-error', $coords['message']);

		return new \WP_REST_Response($coords, 200);
	}

	public function checkout_set_coords($request)
	{

		$parameters = $request->get_params();
		$place = [];

		WC()->frontend_includes();
		WC()->initialize_session();

		if (null === WC()->cart && function_exists('wc_load_cart')) {
			wc_load_cart();
		}

		if ($parameters['set']) {

			$coords = mnsjay_coords_parse($parameters['coords']);
			$place = mnsjay_get_place($coords['lat'], $coords['lng']);

			if (!$place['status'])
				return new \WP_Error('webservice-error', $place['message'], $place);

			// save user location sessions
			WC()->session->set(mnsjay_location_session_key($parameters['checkout']), $parameters['coords']);
		}

		// set location session
		WC()->session->set(mnsjay_location_session_isset_key($parameters['checkout']), $parameters['set']);

		// uncache shipping packages
		foreach (WC()->cart->get_shipping_packages() as $package_key => $package) {
			WC()->session->set('shipping_for_package_' . $package_key, false);
		}

		return new \WP_REST_Response($place, 200);
	}

	public function heatmap_orders_get($request)
	{

		$parameters = $request->get_params();

		$orders = wc_get_orders([
			'limit' => -1,
			'orderby' => 'date',
			'order' => 'DESC',
			'return' => 'ids',
			'status' => ['wc-completed', mnsjay_get_option_order_delivery_status()],
			'date_created' => '>' . (time() - ($parameters['interval'] * DAY_IN_SECONDS)),
		]);

		if (!$orders)
			return new \WP_Error('webservice-error', __('No ordes found in this range', 'mnsjay'));

		$points = mnsjay_get_orders_location($orders);

		return new \WP_REST_Response($points, 200);
	}

	public function postman_delivery($request)
	{

		$parameters = $request->get_params();

		if (mnsjay_get_option_delivery_code_active() === 'yes' and $parameters['delivery_code'] !== mnsjay_get_order_delivery_code($parameters['order_id'])) {
			return new \WP_Error('delivery-code', __('Delivery code is not currect', 'mnsjay'), [
				'order_id' => $parameters['order_id']
			]);
		}

		$postman = mnsjay_get_postman(mnsjay_get_order_postman_id($parameters['order_id']));

		$order = wc_get_order($parameters['order_id']);
		$order->set_status(str_replace('wc-', '', mnsjay_get_option_order_delivery_status()));
		$order->add_order_note(sprintf(__('Order delivered to customer by %s.', 'mnsjay'), $postman->get_fullname()), false);
		$order->save();

		$response = [
			'order_id' => $parameters['order_id']
		];

		return new \WP_REST_Response($response, 200);
	}

	public function postman_return($request)
	{

		$parameters = $request->get_params();

		$order = wc_get_order($parameters['order_id']);
		$order->set_status(str_replace('wc-', '', mnsjay_get_option_order_return_status()));
		$order->add_order_note($parameters['note'], 0, true);
		$order->save();

		$response = [
			'order_id' => $parameters['order_id']
		];

		return new \WP_REST_Response($response, 200);
	}

	public function postman_navigation($request)
	{

		$parameters = $request->get_params();

		$order = wc_get_order($parameters['order_id']);
		$method_key = mnsjay_get_order_shipping_key($order);
		$postman = mnsjay_get_postman(mnsjay_get_order_postman_id($order->get_id()));
		$order_coords = mnsjay_get_order_location($order->get_id(), ($order->has_shipping_address() ? 'shipping' : 'billing'));
		$order_coords = $order_coords ? $order_coords : mnsjay_get_order_location($order->get_id(), 'billing');

		switch ($parameters['type']) {
			case 'live-to-origin':
				$origin = $postman->get_last_coords()['coord'];
				$destination = mnsjay_get_shipping_origin_coords($method_key);
				break;
			case 'live-to-destination':
				$origin = $postman->get_last_coords()['coord'];
				$destination = $order_coords;
				break;
			case 'origin-to-destination':
				$origin = mnsjay_get_shipping_origin_coords($method_key);
				$destination = $order_coords;
				break;
		}

		$response = mnsjay_navigation($origin, $destination);

		if (!$response['status']) {
			return new \WP_Error('webservice-error', $response['message'], [
				'order_id' => $parameters['order_id'],
				'origin' => $origin,
				'destination' => $destination,
				'data' => $response
			]);
		}

		return new \WP_REST_Response($response, 200);
	}

	public function postman_tsp($request)
	{

		$parameters = $request->get_params();

		$postman = mnsjay_get_postman($parameters['postman_id']);

		if ($parameters['origin'] === 'live') {
			$origin = [
				$parameters['origin'] => $postman->get_last_coords()['coord']
			];
		} else {
			$origin = [
				$parameters['origin'] => mnsjay_get_shipping_origin_coords($parameters['origin'])
			];
		}

		if ($parameters['destination'] === 'live') {
			$destination = [
				$parameters['destination'] => $postman->get_last_coords()['coord']
			];
		} else if ($parameters['destination'] === 'last') {
			$destination = [];
		} else {
			$destination = [
				$parameters['destination'] => mnsjay_get_shipping_origin_coords($parameters['destination'])
			];
		}

		$pick_ups = [];

		foreach ($parameters['pick_ups'] as $pick_up) {
			$pick_ups[$pick_up] = mnsjay_get_shipping_origin_coords($pick_up);
		}

		$drop_offs = mnsjay_get_orders_location($parameters['drop_offs']);

		$response = mnsjay_tsp($origin, $drop_offs, $destination);

		if (!$response['status']) {
			return new \WP_Error('webservice-error', $response['message']);
		}

		update_user_meta($parameters['postman_id'], '_mnsjay_tsp_data', $response);

		return new \WP_REST_Response($response, 200);
	}

	public function postman_live($request)
	{

		$parameters = $request->get_params();

		$postman = mnsjay_get_postman($parameters['postman_id']);

		if (!$postman) {
			return new \WP_Error('user-error', __('User not exists', 'mnsjay'));
		}

		if (!$postman->is_postman()) {
			return new \WP_Error('access-error', __('User is not a postman!', 'mnsjay'));
		}

		$time = time();
		$postman->set_last_coords($parameters['coords'], $time);

		$response = [
			'postman_id' => $parameters['postman_id'],
			'coords' => $parameters['coords'],
			'time' => $time,
		];

		return new \WP_REST_Response($response, 200);
	}

	public function postman_settings($request)
	{

		$parameters = $request->get_params();

		$postman = mnsjay_get_postman($parameters['postman_id']);

		if (!$postman->is_postman()) {
			return new \WP_Error('access-error', __('User is not a postman!', 'mnsjay'));
		}

		foreach ($parameters['inputs'] as $key => $value) {
			switch ($key) {
				case '_mnsjay_postman_address':
					$postman->set_address($value);
					break;
				case '_mnsjay_postman_city':
					$postman->set_city($value);
					break;
				case '_mnsjay_postman_coords':
					$postman->set_coords($value);
					break;
				case '_mnsjay_postman_email':
					$postman->set_email($value);
					break;
				case '_mnsjay_postman_firstname':
					$postman->set_firstname($value);
					break;
				case '_mnsjay_postman_lastname':
					$postman->set_lastname($value);
					break;
				case '_mnsjay_postman_phone':
					$postman->set_phone($value);
					break;
				case '_mnsjay_postman_postcode':
					$postman->set_postcode($value);
					break;
				case '_mnsjay_postman_state':
					$postman->set_state($value);
					break;
			}
			do_action('mnsjay_postman_update_setting', $postman, $key, $value);
		}

		$response = [
			'message' => __('Postman data has been updated', 'mnsjay')
		];

		return new \WP_REST_Response($response, 200);
	}
}
