<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

function mnsjay_shipping_method() {

	if (class_exists('MNSJAY_Shipping_Method'))
		return;

	/**
	 * Local shipping method.
	 * 
	 * @since 4.0.0
	 */
	class MNSJAY_Shipping_Method extends WC_Shipping_Method {

		/**
		 * All defined Jaayegah shipping methods.
		 *
		 * @var array
		 */
		public $methods = [];

		/**
		 * Shipping method key (Given from option)
		 *
		 * @var string
		 */
		public $method_key = '';

		/**
		 * Customer cart items.
		 *
		 * @var array
		 */
		public $cart_items;

		/**
		 * Customer cart items total cost.
		 *
		 * @var int
		 */
		public $cart_total;

		/**
		 * Customer cart items total weight.
		 *
		 * @var int
		 */
		public $cart_weight;

		/**
		 * Customer address type.
		 *
		 * @var string
		 */
		public $checkout_type = '';

		/**
		 * Shipping method origin coordinates.
		 *
		 * @var string
		 */
		public $origin_coords = '';

		/**
		 * Customer destination coordinates based onaddress type.
		 *
		 * @var string
		 */
		public $destination_coords = '';

		/**
		 * Distance between shipping origin and customer destination.
		 *
		 * @var int
		 */
		public $distance = '';

		/**
		 * Constructor.
		 *
		 * @param integer $instance_id Instance ID.
		 */
		public function __construct($instance_id = 0) {

			$this->id                 = 'mnsjay_shipping';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = __('Jaayegah shipping', 'mnsjay');
			$this->method_description = __('MNS Jaayegah shipping method.', 'mnsjay');
			$this->methods = mnsjay_get_option_shipping_methods();
			$this->supports = [
				'shipping-zones',
				'instance-settings',
				'instance-settings-modal',
			];

			$this->init();
		}

		/**
		 * Init shipping method settings.
		 *
		 * @access public
		 * @return void
		 */
		function init() {

			// Load the settings API
			$this->init_form_fields();

			$this->init_settings();

			$this->method_key = $this->get_option('method_key', array_key_first($this->methods));
			$this->title = isset($this->methods[$this->method_key]) ? $this->methods[$this->method_key]['default_title'] : $this->method_title;
			$this->cart_total  = isset(WC()->cart) ? WC()->cart->get_cart_contents_total() : 0;
			$this->cart_weight = isset(WC()->cart) ? wc_get_weight(WC()->cart->get_cart_contents_weight(), 'g') : 0;
			$this->cart_items = isset(WC()->cart) ? WC()->cart->get_cart() : [];
			$this->checkout_type = mnsjay_get_ship_to_address_type();
			$this->origin_coords = mnsjay_get_shipping_origin_coords($this->method_key);
			$this->destination_coords = mnsjay_get_current_location($this->checkout_type);
			$this->distance = mnsjay_get_distance($this->origin_coords, $this->destination_coords, mnsjay_get_option_distance_calculation_type());

			add_action('woocommerce_update_options_shipping_' . $this->id, [$this, 'process_admin_options']);
			add_action('mnsjay_update_options_shipping_' . $this->method_key, [$this, 'process_admin_options']);
		}

		/**
		 * Init shipping method option fields.
		 *
		 * @return void
		 */
		public function init_form_fields() {

			$methods_options = [];

			foreach ($this->methods as $method_key => $method_values) {
				$methods_options[$method_key] = $method_values['default_title'];
			}

			$this->instance_form_fields = [
				'method_key' => [
					'title' => __('Method', 'mnsjay'),
					'type' => 'select',
					'description' => __('Choose a Jaayegah method', 'mnsjay'),
					'default' => array_key_first($methods_options),
					'options' => $methods_options,
					'desc_tip' => true,
				]
			];
		}

		/**
		 * Is this method available?
		 *
		 * @param array $package Package.
		 * @return bool
		 */
		public function is_available($package): bool {

			$available = $this->is_enabled();

			if (empty($package)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_active($this->method_key)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_roles($this->method_key)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_location_setted($this->method_key, $this->checkout_type)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_distances($this->method_key, $this->distance)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_weights($this->method_key, $this->cart_weight)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_areas($this->method_key, $this->destination_coords)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_regions($this->method_key, $package)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_products_and_categories($this->method_key, $this->cart_items)) {
				$available = false;
			}

			if (!mnsjay_shipping_check_classes($this->method_key, $this->find_shipping_classes($package))) {
				$available = false;
			}

			if (!mnsjay_shipping_check_carts($this->method_key, $this->cart_total)) {
				$available = false;
			}

			$available = apply_filters('woocommerce_shipping_' . $this->id . '_is_available', $available, $package, $this);
			return apply_filters('mnsjay_shipping_' . $this->method_key . '_is_available', $available, $package, $this);
		}

		/**
		 * Finds and returns shipping classes and the products with said class.
		 *
		 * @param mixed $package Package of items from cart.
		 * @return array
		 */
		public function find_shipping_classes($package) {

			$found_shipping_classes = [];

			foreach ($package['contents'] as $item_id => $values) {
				if ($values['data']->needs_shipping()) {

					$found_class = $values['data']->get_shipping_class();

					if (!isset($found_shipping_classes[$found_class])) {
						$found_shipping_classes[$found_class] = [];
					}

					$found_shipping_classes[$found_class][$item_id] = $values;
				}
			}

			return $found_shipping_classes;
		}

		/**
		 * Called to calculate shipping rates for this method. Rates can be added using the add_rate() method.
		 * 
		 * @param array $package Package array.
		 * @return void
		 */
		public function calculate_shipping($package = []) {

			$cost = mnsjay_calculate_shipping($this->method_key, $this->distance, $this->cart_weight, $this->cart_total, $package);
			$cost = apply_filters('woocommerce_shipping_' . $this->id . '_cost', $cost, $package, $this);

			$label = mnsjay_shipping_label($this->title, $this->method_key, $this->cart_total, $this->distance);
			$label = apply_filters('woocommerce_shipping_' . $this->id . '_label', $label, $package, $this);

			$rate = [
				'meta_data' => [
					'method_key' => $this->method_key,
				],
				'label' => $label,
				'cost' => $cost
			];

			$this->add_rate($rate);
		}
	}

	/**
	 * Legacy method. Do not use
	 * 
	 * @deprecated 4.0.0
	 */
	class MNSJAY_Variable_Shipping_Method extends WC_Shipping_Method {
		/**
		 * Constructor for your shipping class
		 *
		 * @access public
		 * @return void
		 */
		public function __construct() {

			$this->supports = [
				'shipping-zones',
				'instance-settings',
				'instance-settings-modal',
			];

			$this->init();
		}

		/**
		 * Init your settings
		 *
		 * @access public
		 * @return void
		 */
		function init() {

			// Load the settings API
			$this->init_form_fields();

			$this->init_settings();

			$this->title = $this->get_option('title', mnsjay_get_option_shipping_methods()[$this->id]['title']);
			$this->cart_total  = isset(WC()->cart) ? WC()->cart->get_cart_contents_total() : 0;
			$this->cart_weight = isset(WC()->cart) ? wc_get_weight(WC()->cart->get_cart_contents_weight(), 'g') : 0;
			$this->cart_items = isset(WC()->cart) ? WC()->cart->get_cart() : array();

			// Save settings in admin if you have any defined
			add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
		}

		public function init_form_fields() {

			$this->instance_form_fields = [
				'title' => array(
					'title' => __('Title', 'mnsjay'),
					'type' => 'text',
					'description' => __('Title to be display on site', 'mnsjay'),
					'default' => $this->title,
					'desc_tip'    => true,
				),
			];
		}

		public function is_available($package): bool {

			$available = $this->is_enabled();

			if (empty($package)) {
				$available = false;
			}

			$ship_to = mnsjay_get_ship_to_address_type();
			$location = mnsjay_get_current_location($ship_to);
			$state = mnsjay_get_current_state($ship_to);
			$city = mnsjay_get_current_city($ship_to);

			$origin_location = mnsjay_get_option_origin_locations();
			$origin_location[$this->id] = isset($origin_location[$this->id]) ? $origin_location[$this->id] : mnsjay_get_default_location();

			if (!mnsjay_shipping_is_available($this->id)) {
				$available = false;
			}

			if (!mnsjay_shipping_is_in_distance_range($this->id, mnsjay_do_get_distance_haversine($origin_location[$this->id], $location))) {
				$available = false;
			}

			if (!mnsjay_shipping_is_in_weight_range($this->id, $this->cart_weight)) {
				$available = false;
			}

			if (!mnsjay_shipping_is_in_zone_by_check_type($this->id, $location, $state, $city)) {
				$available = false;
			}

			if (!mnsjay_shipping_is_in_products_and_categories($this->id, $this->cart_items)) {
				$available = false;
			}

			if (mnsjay_shipping_is_in_inactive_cart_range($this->id, $this->cart_total)) {
				$available = false;
			}

			return apply_filters('woocommerce_shipping_' . $this->id . '_is_available', $available, $package, $this);
		}

		/**
		 * This function is used to calculate the shipping cost. Within this function we can check for weights, dimensions and other parameters.
		 *
		 * @access public
		 * @param mixed $package
		 * @return void
		 */
		public function calculate_shipping($package = array()) {

			$ship_to = mnsjay_get_ship_to_address_type();
			$location = mnsjay_get_current_location($ship_to);

			$origin_location = mnsjay_get_option_origin_locations();
			$origin_location[$this->id] = isset($origin_location[$this->id]) ? $origin_location[$this->id] : mnsjay_get_default_location();

			$distance = mnsjay_do_get_distance_haversine($origin_location[$this->id], $location);
			$cost = mnsjay_calculate_shipping_cost($this->id, $distance, $this->cart_weight, $this->cart_total);
			$cost = apply_filters('woocommerce_shipping_' . $this->id . '_cost', $cost, $package, $this);

			$label = mnsjay_generate_shipping_label($this->title, $this->id, $this->cart_total, $distance);
			$label = apply_filters('woocommerce_shipping_' . $this->id . '_label', $label, $package, $this);

			$rate = array(
				'id' => $this->id,
				'label' => $label,
				'cost' => $cost
			);

			$this->add_rate($rate);
		}
	}

	class MNSJAY_Delivery_1_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_1';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_1']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 1);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_2_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_2';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_2']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 2);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_3_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_3';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_3']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 3);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_4_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_4';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_4']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 4);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_5_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_5';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_5']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 5);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_6_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_6';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_6']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 6);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_7_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_7';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_7']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 7);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_8_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_8';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_8']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 8);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_9_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_9';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_9']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 9);

			parent::__construct();
		}
	}

	class MNSJAY_Delivery_10_Shipping_Method extends MNSJAY_Variable_Shipping_Method {

		public function __construct($instance_id = 0) {

			$methods = mnsjay_get_option_shipping_methods();

			$this->id                 = 'mnsjay_shipping_10';
			$this->instance_id        = absint($instance_id);
			$this->method_title       = $methods['mnsjay_shipping_10']['title'] . ' (' . __('Deprecated - Do not use', 'mnsjay') . ')';
			$this->method_description = sprintf(__('Delivery %s shipping method (Jaayegah)', 'mnsjay'), 10);

			parent::__construct();
		}
	}
}
add_action('woocommerce_shipping_init', 'mnsjay_shipping_method');

function mnsjay_add_shipping_methods($methods) {

	$methods['mnsjay_shipping'] = 'MNSJAY_Shipping_Method';

	foreach (mnsjay_get_option_shipping_methods() as $method_key => $method_values) {

		if (!mnsjay_is_active($method_values))
			continue;

		$methods[$method_key] = 'MNSJAY_Delivery_' . substr($method_key, strrpos($method_key, '_') + 1) . '_Shipping_Method';
	}

	return $methods;
}
add_filter('woocommerce_shipping_methods', 'mnsjay_add_shipping_methods', 10);
